# /app.py
import os
from flask import Flask, g, redirect, url_for, render_template, flash, request, current_app
from flask_login import login_user, logout_user, login_required, current_user
from config import Config
from apscheduler.schedulers.background import BackgroundScheduler
import atexit
from news_system.orchestrator import run_news_pipeline
from news_system.log_config import setup_logging
from app.models import User
from news_system.db_core import get_app_settings, get_db_connection
from news_system.db_rss import get_or_create_rss_token, regenerate_rss_token
from flask import send_from_directory
from pathlib import Path
from app.extensions import login_manager
import psycopg2
import psycopg2.extras

# --- Logging Setup ---
setup_logging() # Configure logging early

# --- Flask-Login Setup ---
# user_loader needs access to the db context, ensure it's available
@login_manager.user_loader
def load_user(user_id):
    """Loads user object from the database for Flask-Login."""
    try:
        # Use the context manager from db_core
        with get_db_connection() as conn:
            # Use DictCursor for easy access by column name
            cursor = conn.cursor(cursor_factory=psycopg2.extras.DictCursor)
            cursor.execute(
                'SELECT id, username, role, rss_token FROM users WHERE id = %s',
                (int(user_id),) # Ensure user_id is integer
            )
            user_data = cursor.fetchone()
        if user_data:
            return User(id=user_data['id'], username=user_data['username'], role=user_data['role'], rss_token=user_data['rss_token'])
        return None
    except Exception as e:
        # Log error if db access fails during user loading
        try:
            current_app.logger.error(f"Error loading user {user_id}: {e}", exc_info=True)
        except RuntimeError: # Outside of application context
             print(f"Error loading user {user_id} (no app context): {e}")
        return None

# --- Application Factory ---
def create_app(config_class=Config):
    """Creates and configures the Flask application."""
    app = Flask(__name__, instance_relative_config=True) # instance_relative_config=True is good practice
    app.config.from_object(config_class)

    # Ensure the instance folder exists (where the DB resides)
    try:
        os.makedirs(app.instance_path)
    except OSError:
        pass # Already exists

    # Initialize Flask extensions
    login_manager.init_app(app) # Initialize the imported login_manager
    login_manager.login_view = 'auth.login' # Use blueprint name for login route
    login_manager.login_message = 'Please log in to access this page.' # Custom message
    login_manager.login_message_category = 'info'

    # Register Blueprints
    # Import blueprint objects here inside factory to avoid circular imports at top level
    # Ensure blueprints are registered within app context if they need it during setup
    with app.app_context():
        # Core Blueprints
        from app.routes.auth import auth_bp
        from app.routes.user import user_bp
        from app.routes.feed import feed_bp # Import the new feed blueprint
        # Admin Blueprints (New)
        from app.routes.admin_main import admin_main_bp
        from app.routes.admin_agents import admin_agents_bp
        from app.routes.admin_users import admin_users_bp
        from app.routes.admin_config import admin_config_bp
        from app.routes.admin_monitoring import admin_monitoring_bp

        app.register_blueprint(auth_bp)
        # Register all admin blueprints with the '/admin' prefix
        for bp in [admin_main_bp, admin_agents_bp, admin_users_bp, admin_config_bp, admin_monitoring_bp]:
            app.register_blueprint(bp, url_prefix='/admin')
        app.register_blueprint(user_bp)  # No prefix, routes like /my-feed
        app.register_blueprint(feed_bp) # Register feed blueprint (no prefix needed)

    # --- Root Route ---
    @app.route('/')
    def index():
        """Homepage - redirects to login or feed depending on auth status."""
        if current_user.is_authenticated:
            # Redirect to the user feed blueprint route
            return redirect(url_for('user.my_feed'))
        # Redirect to the auth login blueprint route
        return redirect(url_for('auth.login'))

    # --- Scheduler Setup ---
    # Initialize scheduler here to access app context if needed by the job
    scheduler = BackgroundScheduler(daemon=True, timezone='UTC') # Specify timezone

    def schedule_pipeline_job(flask_app):
        """Adds the news pipeline job to the scheduler based on DB settings."""
        # Check if the job already exists to avoid duplicates during reloads
        if not scheduler.get_job('news_pipeline_job'):
            with flask_app.app_context(): # Ensure job runs within app context
                # Fetch the interval from database settings
                settings = get_app_settings()
                interval_minutes = settings.get('pipeline_interval_minutes', 60) # Default to 60 minutes if not found
                if not isinstance(interval_minutes, int) or interval_minutes <= 0:
                    flask_app.logger.warning(f"Invalid pipeline interval '{interval_minutes}' found in settings, defaulting to 60 minutes.")
                    interval_minutes = 60
                flask_app.logger.info(f"Scheduling news pipeline job to run every {interval_minutes} minute(s).")
                scheduler.add_job(
                    run_news_pipeline,
                    trigger='interval',
                    minutes=interval_minutes, # Use minutes instead of hours
                    id='news_pipeline_job',
                    replace_existing=True, # Replace if job exists (e.g., after settings change and restart)
                    misfire_grace_time=600 # Allow 10 mins delay if scheduler missed run
                )
        else:
             flask_app.logger.info("Hourly news pipeline job already scheduled.")

    # Schedule and start only in the main process, not the reloader process
    # WERKZEUG_RUN_MAIN check is crucial for debug mode with reloader
    if not app.debug or os.environ.get('WERKZEUG_RUN_MAIN') == 'true':
        schedule_pipeline_job(app)
        if not scheduler.running:
             try:
                 scheduler.start()
                 # Shut down the scheduler when exiting the app
                 atexit.register(lambda: scheduler.shutdown())
                 app.logger.info("Scheduler started.")
             except Exception as e:
                  app.logger.error(f"Failed to start scheduler: {e}", exc_info=True)

    # Add a simple health check endpoint
    @app.route('/health')
    def health_check():
        # You could add checks here, e.g., database connection
        # For now, just return OK
        return "OK", 200

    # --- Image Serving Route ---
    # Define image directory relative to instance path AFTER app is created
    image_dir = Path(app.instance_path) / "article_images"

    @app.route('/images/<path:filename>')
    def serve_article_image(filename):
        """Serves images stored in the instance/article_images folder."""
        # Use send_from_directory for security
        # Ensure the directory exists (though image_utils should create it)
        if not image_dir.exists():
             app.logger.error(f"Image directory not found: {image_dir}")
             return "Image directory not found.", 404
        try:
            # Security: Basic check to prevent accessing files outside image_dir
            safe_path = Path(image_dir) / filename
            if not safe_path.resolve().is_relative_to(image_dir.resolve()):
                 app.logger.warning(f"Attempt to access file outside image directory: {filename}")
                 return "Forbidden", 403

            return send_from_directory(image_dir, filename)
        except FileNotFoundError:
             app.logger.warning(f"Image file not found: {filename} in {image_dir}")
             return "Image not found.", 404
        except Exception as e:
             app.logger.error(f"Error serving image {filename}: {e}", exc_info=True)
             return "Error serving image.", 500

    return app

# --- Main Execution ---
if __name__ == '__main__':
    flask_app = create_app()
    # Check if DB connection works before running (optional but good)
    try:
        with get_db_connection() as conn:
             print(f"Successfully connected to database.")
    except Exception as e:
        print(f"Database connection failed: {e}. Please ensure PostgreSQL server is running and accessible via: {flask_app.config['DATABASE_URI']}")
    else:
        # Use use_reloader=True for development, but the WERKZEUG_RUN_MAIN check handles scheduler start
        flask_app.run(debug=True, host='0.0.0.0', port=8888) # Host 0.0.0.0 makes it accessible on network
